﻿using SignetTokenTestWeb.Models;
using SignetTokenTestWeb.SignetReference;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Text;
using System.Web;

namespace SignetTokenTestWeb.Helpers
{
    public class SignetHelper
    {
        private string SignetUrl = ConfigurationManager.AppSettings.Get("SignetUrl");
        private string Issuer = ConfigurationManager.AppSettings.Get("SamlIssuer") ?? "prufa.signet.is";
        private string Cert = ConfigurationManager.AppSettings.Get("Cert");
        private string CertPassword = ConfigurationManager.AppSettings.Get("CertPassword");
        private string Account = ConfigurationManager.AppSettings.Get("SignetAccount");
        private string AccountPassword = ConfigurationManager.AppSettings.Get("SignetPassword");
        private string Lang = ConfigurationManager.AppSettings.Get("Lang");
        private bool Auth = (ConfigurationManager.AppSettings.Get("RejectionAuth") ?? "true").Equals("true");

        private string _saml = "";
        private string _token = "";
        private DateTime SamlValidTo = DateTime.Now;
        private DateTime TokenValidTo = DateTime.Now;
        TokenManager manager;

        public SignetHelper()
        {
            manager = new TokenManager(SignetUrl, Issuer, Cert, CertPassword);
        }

        public SignetHelper(string saml, string token)
        {
            manager = new TokenManager(SignetUrl, Issuer, Cert, CertPassword);
            _saml = saml;
            _token = token;
            if (!string.IsNullOrWhiteSpace(token))
                TokenValidTo = manager.GetValidity(token);
        }

        public string GetToken()
        {
            return _token;
        }

        public void SetToken(string token)
        {
            _token = token;
            if (!string.IsNullOrWhiteSpace(token))
                TokenValidTo = manager.GetValidity(token);
        }

        public string GetToken(string ssn, string mobile)
        {
            if (string.IsNullOrWhiteSpace(ssn))
                throw new Exception("SSN missing");
            else if (string.IsNullOrWhiteSpace(Account))
                throw new Exception("Account missing");
            else if (string.IsNullOrWhiteSpace(AccountPassword))
                throw new Exception("Password missing");
            else
            {
                if (string.IsNullOrWhiteSpace(_saml) || SamlValidTo < DateTime.Now)
                {
                    _saml = manager.CreateSaml(Account);
                    SamlValidTo = DateTime.Now.AddHours(1);
                }
                SignetReference.GetTokenRequest request = new SignetReference.GetTokenRequest();                
                request.phone = mobile;
                request.saml = _saml;
                request.ssn = ssn;

                SignetReference.ATokenServiceClient client = new SignetReference.ATokenServiceClient();
                SignetReference.ResultStatus status = null;
                string token = client.GetToken(_saml, GetApiKey(Account, AccountPassword), ssn, mobile, out status);
                if(string.IsNullOrWhiteSpace(token))
                    throw new Exception(status.Message);
                else
                {
                    _token = token;
                    TokenValidTo = manager.GetValidity(_token);
                    return _token;
                }
            }
        }

        public List<DocumentInfov2> GetDocuments(string ssn, string mobile)
        {
            if (string.IsNullOrWhiteSpace(_token) || TokenValidTo < DateTime.Now)
                _token = GetToken(ssn, mobile);
            SignetReference.ATokenServiceClient client = new SignetReference.ATokenServiceClient();
            SignetReference.ResultStatus status = null;
            var docs = client.GetDocumentList(_token, GetApiKey(Account, AccountPassword), out status);

            if (docs == null)
                throw new Exception(status.Message);
            else
            {
                List<DocumentInfov2> _docs = new List<DocumentInfov2>();
                foreach(var doc in docs)
                {
                    DocumentInfov2 _doc = new DocumentInfov2
                    {
                        Added = doc.Added,
                        Archive = doc.Archive,
                        Creator = doc.Creator,
                        Deleted = doc.Deleted,
                        Digest = doc.Digest,
                        DocumentId = doc.DocumentId,
                        DocumentName = doc.DocumentName,
                        DocumentNotes = doc.DocumentNotes,
                        EndSigning = doc.EndSigning,
                        Metadata = doc.Metadata,
                        MininumSigners = doc.MininumSigners,
                        Modified = doc.Modified,
                        Notify = doc.Notify,
                        Ordered = doc.Ordered,
                        Reason = doc.Reason,
                        Signers = doc.Signers,
                        StartSigning = doc.StartSigning,
                        Status = doc.Status
                    };
                    _doc.CanSign = (doc.Status == DocumentInfo.DocumentStatus.New || doc.Status == DocumentInfo.DocumentStatus.InSigning) && doc.Signers.Where(i => i.SSN == ssn && i.HasSigned == false && i.Role == DocumentSigner.SigningRole.Signer).FirstOrDefault() != null;
                    _docs.Add(_doc);
                }
                return _docs.OrderBy(i => i.Added).ToList();
            }                
        }

        public List<byte[]> GetImages(string docId, string ssn, string mobile)
        {
            if (string.IsNullOrWhiteSpace(_token) || TokenValidTo < DateTime.Now)
                _token = GetToken(ssn, mobile);
            SignetReference.ATokenServiceClient client = new SignetReference.ATokenServiceClient();
            SignetReference.ResultStatus status = null;
            var images = client.GetDocumentImages(_token, GetApiKey(Account, AccountPassword), docId, out status);
            if (images == null)
                throw new Exception(status.Message);
            else
                return images.ToList();
        }

        public byte[] GetDocument(string docId, string ssn, string mobile)
        {
            if (string.IsNullOrWhiteSpace(_token) || TokenValidTo < DateTime.Now)
                _token = GetToken(ssn, mobile);
            SignetReference.ATokenServiceClient client = new SignetReference.ATokenServiceClient();
            SignetReference.ResultStatus status = null;
            byte[] document = client.GetDocument(_token, GetApiKey(Account, AccountPassword), docId, out status);
            if (document == null)
                throw new Exception(status.Message);
            return document;
        }

        public string GetDocumentContent(string docId, string ssn, string mobile)
        {
            if (string.IsNullOrWhiteSpace(_token) || TokenValidTo < DateTime.Now)
                _token = GetToken(ssn, mobile);
            SignetReference.ATokenServiceClient client = new SignetReference.ATokenServiceClient();
            SignetReference.ResultStatus status = null;
            string document = client.GetDocumentContent(_token, GetApiKey(Account, AccountPassword), docId, out status);
            if (document == null)
                throw new Exception(status.Message);
            return document;
        }

        public DocumentInfo GetDocumentInfo(string docId, string ssn, string mobile)
        {
            if (string.IsNullOrWhiteSpace(_token) || TokenValidTo < DateTime.Now)
                _token = GetToken(ssn, mobile);
            SignetReference.ATokenServiceClient client = new SignetReference.ATokenServiceClient();
            SignetReference.ResultStatus status = null;
            DocumentInfo info = client.GetDocumentInfo(_token, GetApiKey(Account, AccountPassword), docId, out status);
            if (info == null)
                throw new Exception(status.Message);
            return info;
        }

        public bool DeleteDocument(string docId, string ssn, string mobile)
        {
            if (string.IsNullOrWhiteSpace(_token) || TokenValidTo < DateTime.Now)
                _token = GetToken(ssn, mobile);
            SignetReference.ATokenServiceClient client = new SignetReference.ATokenServiceClient();
            SignetReference.ResultStatus status = null;
            bool success = client.DeleteDocument(_token, GetApiKey(Account, AccountPassword), docId, out status);
            if (!success)
                throw new Exception(status.Message);
            return success;
        }

        public bool SignDocument(string docId, string ssn, string mobile)
        {
            if (string.IsNullOrWhiteSpace(_token) || TokenValidTo < DateTime.Now)
                _token = GetToken(ssn, mobile);
            SignetReference.ATokenServiceClient client = new SignetReference.ATokenServiceClient();
            SignetReference.ResultStatus status = null;
            bool success = client.SignDocument(_token, GetApiKey(Account, AccountPassword), docId, Lang, out status);
            if (!success)
                throw new Exception(status.Message);
            return success;
        }

        public string SignDocument(string docId, string ssn, string mobile, string returnUrl)
        {
            if (string.IsNullOrWhiteSpace(_token) || TokenValidTo < DateTime.Now)
                _token = GetToken(ssn, mobile);
            GetSignTokenRequest request = new GetSignTokenRequest { token = _token, docId = docId, returnUrl = returnUrl };
            SignetReference.ATokenServiceClient client = new SignetReference.ATokenServiceClient();
            SignetReference.ResultStatus status = null;
            string token = client.GetSignToken(_token, GetApiKey(Account, AccountPassword), docId, returnUrl, out status);
            if (string.IsNullOrWhiteSpace(token))
                throw new Exception(status.Message);
            else
                return token;
        }

        public bool RejectDocument(string docId, string ssn, string mobile, string reason)
        {
            if (string.IsNullOrWhiteSpace(_token) || TokenValidTo < DateTime.Now)
                _token = GetToken(ssn, mobile);
            SignetReference.ATokenServiceClient client = new SignetReference.ATokenServiceClient();
            SignetReference.ResultStatus status = null;
            bool success = client.RejectDocument(_token, GetApiKey(Account, AccountPassword), docId, reason, Lang, Auth, out status);
            if (!success)
                throw new Exception(status.Message);
            return success;
        }
        
        public string AddDocument(AddDocumentRequest request, string ssn, string mobile)
        {
            if (string.IsNullOrWhiteSpace(_token) || TokenValidTo < DateTime.Now)
                _token = GetToken(ssn, mobile);            
            SignetReference.ATokenServiceClient client = new SignetReference.ATokenServiceClient();
            SignetReference.ResultStatus status = null;
            string id = client.AddDocument(_token, GetApiKey(Account, AccountPassword), request, out status);
            if (string.IsNullOrWhiteSpace(id))
                throw new Exception(status.Message);
            else
                return id;
        }

        private string GetApiKey(string user, string pass)
        {
            return Convert.ToBase64String(Encoding.UTF8.GetBytes(user + ":" + pass));
        }
    }
}